
## 14/09/2018 - Script for biomod2 Species Distribution Models (SDMs) ##
## Sarah A. Scriven ##

## 1. Install packages ##
library (raster)
library (sp)
library (rgdal)    
library (maptools) 
library (biomod2)
library (grDevices) # For saving plots


###################################
## Sorting the climate variables ##
###################################

## Nb: Climate variables for 1970-2000 were downloaded dircetly from the Worldclim database (http://worldclim.org/version2 [accessed June 9th 2018]) at ~1 km2 (30 arc-second) resolution
## and processed using the "WorldclimData_CorrelationScript.R" Script contained in the "SDM_Analyses.zip" folder.

## 2. Upload the five selected climate variables (nb: climate rasters are not provided in archived dataset) ## 

setwd ("G:/Rainforest Trust Project/SDM Biotropica Manuscript/BioclimVariables1km/SDMVariables5_1kmUTM50N")
AnnualMeanTemp<- raster("AnnualMeanTempBorneo1km.tif") # 'Annual mean temperature (C)' (bio1)
MeanDiurnalRange<- raster ("MeanDiurnalRangeBorneo1km.tif") # 'Mean diurnal temperature range (C)' (bio2)
Seasonality<- raster("TemperatureSeasonalityBorneo1km.tif") # 'Temperature seasonality (C)' (bio4)
PrecipDriestMonth<- raster("PrecipDriestMonthBorneo1km.tif") # 'Precipitation of the driest month of the year (mm)' (bio14)
PrecipWettestQuarter<- raster("PrecipWettestQuarterBorneo1km.tif") # 'Precipitation of the wettest quarter of the year (mm)' (bio16)

## 3. Stack the climate variables to run in Biomod2 ##
myExpl<- stack(AnnualMeanTemp, MeanDiurnalRange, Seasonality, PrecipDriestMonth, PrecipWettestQuarter)
plot (AnnualMeanTemp) 
plot (MeanDiurnalRange)
plot (Seasonality)
plot (PrecipDriestMonth)
plot (PrecipWettestQuarter) 

################
## Sabah Plot ##
################

## 4. Load in Sabah boarder shapefile for cropping in loop after SDMs have been run ##
# Nb: Sabah boarder shapefile can be obtained from: https://gadm.org/maps.html; e.g., 
# mal.shp <- getData('GADM', country='MYS',level=1) # Malaysia shape file 
# Sab<-mal.shp[mal.shp@data$NAME_1=="Sabah",] # Sabah shape file 

setwd("G:/SDMsTrial")
mal.shp<- readRDS("GADM_2.8_MYS_adm1.rds", refhook = NULL) 
Sab<-mal.shp[mal.shp@data$NAME_1=="Sabah",]
SabP<- spTransform(Sab, CRS("+proj=utm +zone=50 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs")) # Project Sabah shapefile
plot (SabP)

############################################
## Setting up the butterfly locality data ##
############################################

## 5. Load in butterfly data and extract sample species ##
# The raw B2D2 butterfly database is available to download from: http://www-users.york.ac.uk/~jkh6/index.htm 
# Archived butterfly data for this study has been extracted for Borneo and only includes unique records post 1950 (i.e., any duplicated records of the same species at the same locality have been removed)
# The uploaded data includes all types of species (including those not restricted to Sundaland, as all localities will be used for 'absences' in the SDMs)

setwd("G:/Rainforest Trust Project/SDM Biotropica Manuscript/SDMs_Biotropica1kmLooped")
butterflydata<-read.csv("ButterflyDataBorneo_UniqueRecordsSITE1950s.csv", header =T)  # Butterfly data for Borneo which includes all unique records post 1950. 									              
summary (butterflydata) # Check the raw data
View (butterflydata)

# LAT = Latitude (Y)
# LONG = Longitude (X)
# ORIGNUM = Original number of record in B2D2 database
# SECIES = Species name
# ENDEMIC = 'Endemic' = endemic to Borneo; 'Sundaland' = found only in Sundaland; 'NonEndemic' = found outside of Sundaland
# FAMILY = Butterfly family 
# LOCALITY = Location where record was collected
# PROVINCES = Province where record was collected
# YEAR = Year collected 
# Site = Site number; a unique number given to each individual locality based on the X/Y co-ordinates (398 unique sampling locations in total)
# See: http://www-users.york.ac.uk/~jkh6/index.htm for more details about the raw data used

butterflydata$Y<- butterflydata$LAT
butterflydata$X<- butterflydata$LONG

#########################
## Setting up the loop ##
#########################

## 6. Extract the 'Endemic' and 'Sundaland' species (just to get species number 1-78) and add 'Species number' to full data set ##
Endemics<-butterflydata[butterflydata$ENDEMIC=="Endemic"|butterflydata$ENDEMIC=="Sundaland",]
SPECIES<- unique (Endemics$SPECIES)
length (SPECIES) # 78 - run models for 77 (don't run models for number 78: Papilio iswaroides, as only one locality record)
Endemics78<- data.frame (SPECIES)
Endemics78$NUMBER<- 1:78
write.csv (Endemics78, "ButterflySpeciesNumbers.csv") # Found in the 'SDM_OutputMaps' folder; useful for interpretting output maps which are labelled by this number

butterflydata$SPECIESNO<-NA # Full dataframe 
butterflydata$SPECIESNO <-Endemics78$NUMBER[match(butterflydata$SPECIES, Endemics78$SPECIES)] # Only Endemic/Sundaland species are given a number
head (butterflydata, n=100)


#######################################################################################################################################################################################
## Run loop from here ##
########################

## 7. Set up Matrix for AUC model results ##
AUCmatrix<- matrix (NA,78,4, dimnames=list(NULL, c("SpeciesNo", "Mean", "Min", "PoorModels"))) # Make a matrix outside the loop to record the mean and min AUC values, and the number of models with an AUC <0.6 (i.e., 'Poor models')
AUCmatrix [,1]<-1:78

for (s in 1:77){ # Use ALL the raw data for absences, i.e., for ALL species not just the Sundaland butterflies, but run the SDMs for the 77 Sundaland-restricted butterflies 

		## 8. Set up the dataframe for the chosen species ##
		setwd("G:/Rainforest Trust Project/SDM Biotropica Manuscript/SDMs_Biotropica1kmLooped")
		ChosenSpecies<- butterflydata[butterflydata$SPECIESNO==s,] ## Extract species from 'butterflydata' 
		ChosenSpecies<- ChosenSpecies[complete.cases (ChosenSpecies),]
		#length (ChosenSpecies$X) # e.g., 14 unique locality records for 'species 1'

		pointsChosenSpecies<- cbind(ChosenSpecies$X, ChosenSpecies$Y) 
		plot (pointsChosenSpecies, col="blue") # Check that data look ok
		dir.create(paste0("Endemicspecies_",s))
		setwd(paste0("Endemicspecies_",s))
		savePlot (paste0(filename="pointsChosenSpecies",s),type= "jpeg", device=dev.cur()) # Save the response plot

		## 9. Species selection ##
		butterflydata$PRS<-0 # Create a column called PRS 
		butterflydata$PRS[butterflydata$SPECIESNO==s]<-1 # Code column 'PRS' with 1 if species 's' is present, otherwise 0 represents an absence
		#View (butterflydata)

		## 10. Remove all overlapping localities irrespective of species (the data set currently has rows for the same sites representing different species) ##
		dups <- duplicated(butterflydata[, c('X', 'Y')]) # Finds duplicates based on X/Y coordinates (i.e., the same locality)
		#View (dups) # TRUE or FALSE
		#sum(dups) # Number of duplicates (this will change depending on the species); e.g., 7263 for 'species 1'
		butterflydata$dups<-dups
		Noduplicates <- butterflydata[!dups, ] # Keep the records that are not duplicated
		#View (Noduplicates)
		length (Noduplicates$X) # Always 398 unique sampling locations on Borneo post 1950
		
		ChosenSpecies<- butterflydata[butterflydata$SPECIESNO==s,] # Extract target species rows from 'butterflydata' 
		Modeldata<-merge(x=ChosenSpecies,y=Noduplicates, by=c("X", "Y"), all.y=TRUE) ## Then merge dataframes to form model data with no duplicates 
		#View (Modeldata)
		#sum (Modeldata$PRS.x, na.rm=TRUE) # Number of presence localities - should be the same as above
		Modeldata$PRS<-Modeldata$PRS.x
		length (Modeldata$PRS) # 398 - should also match above
		#write.csv (Modeldata, file =paste ("ModeldataChosenSpecies", s, 'csv', sep='.')) # Write to csv file if needed

		## 11. Merge datasets for SDM ##
		#head (Modeldata)
		SDMdata<- Modeldata[,c("X","Y","PRS","Site.y")] # Select these columns for inclusion
		SDMdata[ ,3][is.na(SDMdata[ ,3] ) ] = 0 # Change NAs in the column to 0s (represents absences)
		#head (SDMdata)
		names (SDMdata) [4]<- paste ("Site")
		#write.csv (SDMdata, file =paste ("SDMdataSpecies", s, 'csv', sep='.')) # Write to csv file if needed

		## Hence we are left with a dataframe 'SDMdata' that contains all the presences of the focal species (with any duplicates removed)
		## and the localities where other species were found; i.e. that will be used as 'absences'.

		## 12. Set up the XY Coordinates ## 
		X<- as.numeric (SDMdata$X) 
		Y<- as.numeric (SDMdata$Y)
		XY<-cbind (c(X), (Y))
		XYP<- project(XY, "+proj=utm +zone=50 +datum=WGS84 +units=m +no_defs++ellps=WGS84 +towgs84=0,0,0") # Project the X/Y coordinates into m 
		#class (XYP) # Check XY coordinates

		myRespXY<-data.frame (XYP)
		names (myRespXY) [1]<- paste ("X")
		names (myRespXY) [2]<- paste ("Y")
		SDMdata$XP<- myRespXY$X
		SDMdata$YP<- myRespXY$Y
		#plot (SDMdata$XP, SDMdata$YP)


		#####################################
		## Setting up BIOMOD_FormatingData ##
		#####################################

		## 13. Setting up the biomod2 models ##
		#names (SDMdata) # Recall the names in the 'SDMdata' dataframe
		myRespName <- 'PRS' # This name should be exactly be same as the column name of the presence-absence data
		myResp <- as.numeric(SDMdata[,myRespName]) # Vector of 1's representing presences
		XYP<-cbind (c(SDMdata$XP), (SDMdata$YP))
		myRespXY<-data.frame (XYP) # Should also be in m
		names (myRespXY) [1]<- paste ("X")
		names (myRespXY) [2]<- paste ("Y")

		# 'myBiomodData' - This sorts out all the data into biomod2 format
		#?BIOMOD_FormatingData

		myBiomodData <- BIOMOD_FormatingData(resp.var = myResp,   # Response variable
                                     expl.var = myExpl,   # Explanatory variables - the raster stack
                                     resp.xy = myRespXY,  # X/Y data
                                     resp.name = myRespName, # Name of response
                                     na.rm=TRUE)
		myBiomodData
		plot(myBiomodData) 

		## 14. Defining the SDMs that you wish to run ##
			myBiomodOption <- BIOMOD_ModelingOptions(
  			MAXENT.Phillips = list(path_to_maxent.jar = "G:/SDMsTrial/maxent", # You need to have MAXENT.jar in your working directory
                         	memory_allocated = 1024,      # Det memory - 1024 is the default
                         	maximumiterations = 200,      # Num iterations - may need to increase this in order for the model to converge. 200 is the default
                         	visible = FALSE,              # Do you want the maxent interface visible?
                         	linear = TRUE,                # Allows linear features to be used (use linear coefficients for each predictor)
                         	quadratic = TRUE,             # Quadratic features use squared predictor values
                         	product = FALSE,              # Product features describe pairwise interactions between predictors
                         	threshold = FALSE,            # Threshold features involve simple step functions
                         	hinge = FALSE,                # Hinge features (i.e., piecewise linear splines) combine linear and step functions
                         	lq2lqptthreshold = 80,        # Integer (default 80), number of samples at which product and threshold features start being used
                         	l2lqthreshold = 10,           # Integer (default 10), number of samples at which quadratic features start being used
                        	hingethreshold = 15,          # Integer (default 15), number of samples at which hinge features start being used
                        	beta_threshold = -1, 	      # Numeric (default -1.0), regularization parameter to be applied to all threshold features; negative value enables automatic setting
                        	beta_categorical = -1, 	      # beta_categorical: numeric (default -1.0), regularization parameter to be applied to all categorical features; negative value enables automatic setting
                        	beta_lqp = -1,		      # beta_lqp: numeric (default -1.0), regularization parameter to be applied to all linear, quadratic and product features; negative value enables automatic setting
                         	beta_hinge = -1,	      # numeric (default -1.0), regularization parameter to be applied to all hinge features; negative value enables automatic setting
			 	#betamultiplier = 1	      # numeric (default 1), multiply all automatic regularization parameters by this number. A higher number gives a more spread-out distribution
                         	defaultprevalence = 0.5),     # numeric (default 0.5), default prevalence of the species: probability of presence at ordinary occurrence points
  
  			GLM = list(type = 'simple',                        # This can be 'simple', 'quadratic' or 'polynomial'; 'qaudratic' is the default
            			interaction.level = 0,                     # Whether you want the model to condiser interactions...note that if this >0, this will enlarge the number of effective variables in the model 
            			myFormula = NULL, 				  
           			test = 'AIC',                              # Stepwise selection based on AIC
           			family = binomial(link = 'logit'),         # The error distribution and link function
            			mustart = 0.5,				   # Default
            			control = glm.control(epsilon = 1e-08, maxit = 50, trace = FALSE)), # A list of parameters controlling the fitting process
  
  			RF = list (do.classif = TRUE, # If TRUE, classification random.forest computed, else regression random.forest will be done
		 			ntree = 501, # Default = 500 (but best to have an odd number so ties can be broken)
		 			mtry = 'default', 
		 			nodesize = 5, # Default = 5; this represents the smallest node which can be split, so when you increase the node size you will grow smaller trees (and lose predictive power)
		 			maxnodes = NULL)) # Don't set a maximum number of nodes


			## 15. Running the models ## This may take a while depending on the resolution... 
			myBiomodModelOut <- BIOMOD_Modeling(
 			myBiomodData, # Your data 
  			models = c('MAXENT.Phillips', 'GLM', 'RF'), # The models you want to run
  			models.options = myBiomodOption, # The options for those models  
  			NbRunEval=10, # Number of times you want to run each model
  			DataSplit=75, # Split the data into 75% and 25% for calibrating and training, and testing, respectively
  			#Yweights=NULL, 
  			Prevalence=0.5, # So the presences and absences are weighted equally
  			VarImport=3, # Number of permutations to estimate variable importance
  			models.eval.meth = 'ROC', # A measure of model performance (ROC: recieving operating characteristic)
  			SaveObj = TRUE,
 			rescal.all.models = TRUE,
  			do.full.models = FALSE) # If true, models calibrated and evaluated with the whole dataset 

			#####################
			## The SDM Outputs ##
			#####################

			## 16. Saving model outputs ##
			#myBiomodModelOut # Look at model output/evaluation; also note that new directories have been made in the working directory
			warnings()  

			eval <- get_evaluations(myBiomodModelOut,as.data.frame=TRUE) # This will give you model performance
			#View(eval) # Check to see if any models failed
			write.csv (eval, file =paste ("EvaluationTableSpecies", s, 'csv', sep='.'))
		
			evaldf<-data.frame (eval)
			#names (evaldf)
			meanAUC<- mean(evaldf$Testing.data) # Mean AUC value across models
			minAUS<- min(evaldf$Testing.data) # Min AUC value across models
			Poormodels<-evaldf[evaldf$Testing.data<=0.6,] # Go for a 0.6 threshold following Cheng & Bonebreak (2017)
			lengthAUC<-length (Poormodels$Testing.data) 
			AUCmatrix [s,]<- c(s, meanAUC,minAUS,lengthAUC) # Put these values into 'AUCmatrix' so that you can compare models across runs 

			var.imp <- get_variables_importance(myBiomodModelOut,as.data.frame=TRUE) # Look at different models scores:
			#View(var.imp)
			write.csv(var.imp, file =paste ("VariableImportanceSpecies", s, 'csv', sep='.')) # This gives the relative importance of each variable in predicting the projected butterfly distribution

			## 17. Plot response curves ##
			# Load the models for which we want to extract the predicted response curves:
			myMAXENT <- BIOMOD_LoadModels(myBiomodModelOut, models='MAXENT.Phillips')
			myGLM <- BIOMOD_LoadModels(myBiomodModelOut, models='GLM')
			myRF <- BIOMOD_LoadModels(myBiomodModelOut, models='RF')

			## 18. Plot 2D response plots ##
		
			### Maxent ### 
			myRespPlot2DMaxent <- response.plot2(models = myMAXENT,
                               Data = get_formal_data(myBiomodModelOut,'expl.var'),
                               show.variables= get_formal_data(myBiomodModelOut,'expl.var.names'),
                               do.bivariate = FALSE,
                               fixed.var.metric = 'median',
                               col = c("blue"),
                               legend = TRUE,
                               data_species = get_formal_data(myBiomodModelOut,'resp.var'))

			#myRespPlot2DMaxent
			write.csv(myRespPlot2DMaxent, file =paste ("ResponsePlotData_MaxentSpecies", s, 'csv', sep='.'))
			savePlot (paste0(filename="ResponsePlot_MaxentSpecies",s), type= "jpeg", device=dev.cur()) # Save the response plot
		
			# These plots will show you the relationship between the predictor variable (x-axis) and probability of species presence (y-axis)
			# The lines in each plot correspond to the 10 model runs

			### GLM ###
			myRespPlot2DGLM <- response.plot2(models = myGLM,
                               Data = get_formal_data(myBiomodModelOut,'expl.var'),
                               show.variables= get_formal_data(myBiomodModelOut,'expl.var.names'),
                               do.bivariate = FALSE,
                               fixed.var.metric = 'median',
                               col = c("red"),
                               legend = TRUE,
                               data_species = get_formal_data(myBiomodModelOut,'resp.var'))
			#myRespPlot2DGLM
			write.csv(myRespPlot2DGLM, file =paste ("ResponsePlotData_GLMSpecies", s, 'csv', sep='.'))
			savePlot (paste0(filename="ResponsePlot_GLMSpecies",s), type= "jpeg", device=dev.cur()) # Save the response plot

			### RF ###
			myRespPlot2DRF <- response.plot2(models = myRF,
                               Data = get_formal_data(myBiomodModelOut,'expl.var'),
                               show.variables= get_formal_data(myBiomodModelOut,'expl.var.names'),
                               do.bivariate = FALSE,
                               fixed.var.metric = 'median',
                               col = c("green"),
                               legend = TRUE,
                               data_species = get_formal_data(myBiomodModelOut,'resp.var'))
			#myRespPlot2DRF
			write.csv(myRespPlot2DRF, file =paste ("ResponsePlotData_RFSpecies", s, 'csv', sep='.'))
			savePlot (paste0(filename="ResponsePlot_RFSpecies",s), type= "jpeg", device=dev.cur()) # Save the response plot

			## 19. Project the individual calibrated models into geographical space ##
			myBiomodProj <- BIOMOD_Projection(modeling.output = myBiomodModelOut,
                                  new.env = myExpl,        # Explanatory variables onto which models will be projected
                                  proj.name = 'current',   # A character defining the projection name (a new folder will be created with this name)
                                  selected.models = 'all', # Which models to use 
                                  binary.meth = 'ROC',     # Model evaluation method 
                                  compress = 'xz',
                                  build.clamping.mask = TRUE, 
                                  do.stack=FALSE,
                                  output.format='.img')
			#myBiomodProj

			## 20. The ensemble part ##

			#View (eval)
			myBiomodEM <- BIOMOD_EnsembleModeling(modeling.output = myBiomodModelOut,
                                       chosen.models = 'all',
                                       em.by = 'all',
                                       eval.metric = c('ROC'),
                                       eval.metric.quality.threshold = c(0.6), # Selecting all model runs with ROC > 0.6 
                                       models.eval.meth = c('ROC'),
                                       prob.mean = TRUE, # Estimate the mean probabilities across predictions
                                       prob.cv = TRUE, # Estimate the coefficient of variation across predictions
                                       prob.ci = FALSE, # Estimate the confidence interval around the prob.mean; set this to false
                                       prob.ci.alpha = 0.05, # Significance level for estimating the confidence interval
                                       prob.median = TRUE, # Estimate the median of probabilities
                                       committee.averaging = TRUE, # Estimate the committee averaging across predictions; see: https://rdrr.io/cran/biomod2/man/BIOMOD_EnsembleModeling.html
                                       prob.mean.weight = TRUE, # Estimate the weighted sum of probabilities (weighted by the best models?)
                                       prob.mean.weight.decay = 'proportional') # If prob.mean.weight = TRUE, then the attributed weights are proportional to the evaluation scores given by the weight.method
			#myBiomodEM
			ens_eva <- get_evaluations(myBiomodEM,as.data.frame=TRUE)
			#View (ens_eva)
			write.csv(ens_eva, file =paste ("EnsembleEvaluationTableSpecies", s, 'csv', sep='.'))

			myBiomodEF<-BIOMOD_EnsembleForecasting(projection.output = myBiomodProj, # This function uses projections of individual models and ensemble models from BIOMOD_EnsembleModeling to build an ensemble of species' projections over space and time
                                        EM.output = myBiomodEM,
                                        binary.meth = 'ROC') 

		
			## 21. Plot the final maps ##
		
			par(mfrow=c(2,1))
			currentensmeblemean<-raster(paste0(getwd(),"/PRS/proj_current/individual_projections/prs_EMmeanByROC_mergedAlgo_mergedRun_mergedData.grd"))
			plot(currentensmeblemean/1000)                            

			currentensmeblemean.bin<-raster(paste0(getwd(),"/PRS/proj_current/individual_projections/prs_EMmeanByROC_mergedAlgo_mergedRun_mergedData_ROCbin"))
			plot(currentensmeblemean.bin)    # Created from the mean of the cutoff values /1000 - found in the 'eval' table above. 
			savePlot (paste0(filename="FinalMapsSpecies",s), type= "jpeg", device=dev.cur()) # Save the response plot

			## 22. Create Tif layers of the output maps ##
			setwd("D:/Rainforest Trust Project/SDM Biotropica Manuscript/SDMs_Biotropica1kmLooped")
			dir.create("FinalMaps")
			setwd("D:/Rainforest Trust Project/SDM Biotropica Manuscript/SDMs_Biotropica1kmLooped/FinalMaps")
			
			writeRaster(currentensmeblemean/1000, paste("EnsembleMeanProbability." ,s, ".tif", sep=""), overwrite=TRUE)
			writeRaster(currentensmeblemean.bin, paste("EnsembleMeanBinaryROC." ,s, ".tif", sep=""), overwrite=TRUE)
	
			EnsemProb <-raster (paste("EnsembleMeanProbability.",s, ".tif", sep=""))
			#plot (EnsemProb) 
			EnsemBin <-raster (paste("EnsembleMeanBinaryROC.",s, ".tif", sep=""))
			#plot (EnsemBin) 

			## 23. Create the final Tif layers - Cropped for Sabah and at 1 km resolution ## 
			EnsemProbSabah<- mask (EnsemProb, SabP)
			EnsemProbSabah<- crop (EnsemProbSabah, SabP)

			newproj<- "+proj=utm +zone=50 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs"
			EnsemProbSabah1km <- projectRaster (EnsemProbSabah, crs=newproj, res=1000)
			EnsemProbSabah1km
			par(mfrow=c(1,1))
			plot (EnsemProbSabah1km)
			savePlot (paste0(filename="EnsemProbSabah1km",s), type= "jpeg", device=dev.cur())

			setwd("D:/Rainforest Trust Project/SDM Biotropica Manuscript/SDMs_Biotropica1kmLooped")
			dir.create("Sabah1kmSDMMaps")
			setwd("D:/Rainforest Trust Project/SDM Biotropica Manuscript/SDMs_Biotropica1kmLooped/Sabah1kmSDMMaps")	
			writeRaster(EnsemProbSabah1km, paste("EnsemProbSabah1kmSpecies.", s, ".tif", sep=""), overwrite=TRUE)
} # End of loop


############################################################################################################################################################################################

## 24. Save the AUC matrix ##
setwd("D:/Rainforest Trust Project/SDM Biotropica Manuscript/SDMs_Biotropica1kmLooped")
AUCmatrix
write.csv (AUCmatrix, "AUCmatrix_Version5.csv")
